package dsl.datamodel.scoping

import java.util.List
import org.eclipse.emf.ecore.EObject
import org.eclipse.emf.ecore.EReference
import org.eclipse.xtext.naming.QualifiedName
import org.eclipse.xtext.scoping.IScope
import org.eclipse.xtext.scoping.Scopes

import java.util.ArrayList
import dsl.datamodel.datamodel.Entity
import dsl.datamodel.datamodel.DatamodelPackage
import dsl.datamodel.datamodel.Context

/**
 * This class contains custom scoping description.
 * 
 * See https://www.eclipse.org/Xtext/documentation/303_runtime_concepts.html#scoping
 * on how and when to use it.
 */
class DatamodelScopeProvider extends AbstractDatamodelScopeProvider {

	val ePackage = DatamodelPackage.eINSTANCE;

	override getScope(EObject context, EReference reference) {
		if (reference == ePackage.entity_ParentEntity && context instanceof Entity) {
			val entity = context as Entity;
			return scopeForParentEntity(entity, context, reference)
		}
		return super.getScope(context, reference)
	}

	def protected IScope scopeForParentEntity(Entity entity, EObject context, EReference reference) {
		val container = entity.eContainer as Context;
		val List<Entity> entities = newArrayList;
		setEntities(container, entities);
		return Scopes::scopeFor(entities, [
			QualifiedName.create((it.eContainer as Context).name.split("\\.")).append(it.name)
		], IScope.NULLSCOPE);
	}

	def protected void setEntities(Context context, List<Entity> entities) {
		entities.addAll(context.entities);
		if (context.parentContext !== null && !context.getName.equals(context.parentContext.getName)) {
			setEntities(context.parentContext, entities);
		}
	}
}
