package dsl.datamodel.scoping;

import com.google.common.base.Function;
import com.google.common.base.Objects;
import dsl.datamodel.datamodel.Context;
import dsl.datamodel.datamodel.DatamodelPackage;
import dsl.datamodel.datamodel.Entity;
import dsl.datamodel.scoping.AbstractDatamodelScopeProvider;
import java.util.List;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.xtext.naming.QualifiedName;
import org.eclipse.xtext.scoping.IScope;
import org.eclipse.xtext.scoping.Scopes;
import org.eclipse.xtext.xbase.lib.CollectionLiterals;

/**
 * This class contains custom scoping description.
 * 
 * See https://www.eclipse.org/Xtext/documentation/303_runtime_concepts.html#scoping
 * on how and when to use it.
 */
@SuppressWarnings("all")
public class DatamodelScopeProvider extends AbstractDatamodelScopeProvider {
  private final DatamodelPackage ePackage = DatamodelPackage.eINSTANCE;
  
  @Override
  public IScope getScope(final EObject context, final EReference reference) {
    if ((Objects.equal(reference, this.ePackage.getEntity_ParentEntity()) && (context instanceof Entity))) {
      final Entity entity = ((Entity) context);
      return this.scopeForParentEntity(entity, context, reference);
    }
    return super.getScope(context, reference);
  }
  
  protected IScope scopeForParentEntity(final Entity entity, final EObject context, final EReference reference) {
    EObject _eContainer = entity.eContainer();
    final Context container = ((Context) _eContainer);
    final List<Entity> entities = CollectionLiterals.<Entity>newArrayList();
    this.setEntities(container, entities);
    final Function<Entity, QualifiedName> _function = (Entity it) -> {
      EObject _eContainer_1 = it.eContainer();
      return QualifiedName.create(((Context) _eContainer_1).getName().split("\\.")).append(it.getName());
    };
    return Scopes.<Entity>scopeFor(entities, _function, IScope.NULLSCOPE);
  }
  
  protected void setEntities(final Context context, final List<Entity> entities) {
    entities.addAll(context.getEntities());
    if (((context.getParentContext() != null) && (!context.getName().equals(context.getParentContext().getName())))) {
      this.setEntities(context.getParentContext(), entities);
    }
  }
}
